"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const get = require("lodash/get");
const isFunction = require("lodash/isFunction");
const module_1 = require("./module");
const utils_1 = require("./utils");
/**
 * A simple module to quickly convert nodejs module to Magnet module
 * @param  {[function, object]} module      Module to be converted
 * @param  {string} options.namespace   Namespace to occupy, same for app[namespace], config[namespace]
 * @param  {string} options.initializer Initialize function
 * @param  {[array, function]} options.params      Will pass to initializer
 * @return void
 */
function convert(module, convertOptions, defaultConfig = {}) {
    const { initializer, params, teardown } = convertOptions;
    const moduleName = convertOptions.namespace;
    return class ConvertMagnetModule extends module_1.Module {
        // Set class name
        // http://stackoverflow.com/a/41787315/788518
        // static get name () { return namespace }
        constructor(app, options = {}) {
            super(app, options);
            this.name = moduleName;
        }
        setup() {
            return __awaiter(this, void 0, void 0, function* () {
                const config = this.prepareConfig(moduleName, defaultConfig);
                // Prepare parameters
                let moduleParams = [];
                if (isFunction(params)) {
                    moduleParams = [params(config)];
                }
                else if (Array.isArray(params)) {
                    for (const param of params) {
                        if (param.startsWith('config.')) {
                            const configKey = param.replace('config.', '');
                            moduleParams.push(get(config, configKey));
                        }
                        else if (param === 'config') {
                            moduleParams.push(config);
                        }
                    }
                }
                else {
                    this.log.warn('Params is not recognized');
                }
                const initialize = module[initializer] || module;
                if (utils_1.isClass(initialize)) {
                    this.app[moduleName] = new initialize(...moduleParams);
                }
                else {
                    this.app[moduleName] = initialize(...moduleParams);
                }
            });
        }
        teardown() {
            return __awaiter(this, void 0, void 0, function* () {
                if (teardown && this.app[moduleName] && this.app[moduleName][teardown]) {
                    this.app[moduleName][teardown]();
                    this.log.info(`${moduleName} teardown completed`);
                }
            });
        }
    };
}
exports.default = convert;
//# sourceMappingURL=convert.js.map